@@ -145,7 +145,8 @@ class PreScan(VisitorBase):
         self, ctx: qp.RegularQuerySpecificationContext
     ) -> None:
         self.assert_none(
-            ctx.lateralView(), ctx.windowClause(),
+            ctx.lateralView(),
+            ctx.windowClause(),
         )
         if ctx.fromClause() is not None:
             self.copy(PreFrom).visit(ctx.fromClause())
@@ -513,7 +514,7 @@ class OrganizationVisitor(VisitorBase):
             is_col, _ = PreFunctionArgument(self).visit_argument(ctx.limit)
             self.assert_support(not is_col, ctx)
             limit = int(eval(self.to_str(ctx.limit)))
-            self.assert_support(limit > 0, ctx)
+            self.assert_support(limit >= 0, ctx)
         else:
             limit = -1
         return order_by, limit
@@ -559,6 +560,13 @@ class ExpressionVisitor(VisitorBase):
     ) -> List[Column]:
         return self.visit(ctx.expression())
 
+    def visitArithmeticUnary(  # type: ignore
+        self, ctx: qp.ArithmeticUnaryContext
+    ) -> List[Column]:
+        op = self.to_str(ctx.operator)
+        v = self._get_single_column(ctx.valueExpression())
+        return [self.workflow.op_to_col("basic_unary_arithmetic_op", v, op)]
+
     def visitArithmeticBinary(  # type: ignore
         self, ctx: qp.ArithmeticBinaryContext
     ) -> List[Column]:
@@ -695,7 +703,7 @@ class FromVisitor(ExpressionVisitor):
             on: List[str] = []
             if len(join.conditions) > 0:
                 for i in range(len(join.conditions)):
-                    join_col_name = f"_{i}"
+                    join_col_name = f"_{i}_"
                     left_cols.append(
                         self._join_col(join.conditions[i][0], joined).rename(
                             join_col_name
@@ -704,7 +712,7 @@ class FromVisitor(ExpressionVisitor):
                     right_cols.append(
                         self._join_col(join.conditions[i][1], df).rename(join_col_name)
                     )
-                    on.append(f"_{i}")
+                    on.append(f"_{i}_")
                 left = WorkflowDataFrame(joined, *left_cols)
                 right = WorkflowDataFrame(df, *right_cols)
             joined = self.workflow.op_to_df(
