@@ -34,8 +34,9 @@ class QPDEngineTests(object):
                     second = df[second]
                 else:
                     second = Column(second)
-                res = self.op.comparison_op(first, second, op).native.tolist()
-                assert_df_eq(pd.DataFrame({"a": res}), pd.DataFrame({"a": expected}))
+                self.assert_column_eq(
+                    self.op.comparison_op(first, second, op), expected
+                )
 
             assert self.op.comparison_op(Column(0), Column(1), "<").native
             assert not self.op.comparison_op(Column(0), Column(1), ">").native
@@ -77,7 +78,9 @@ class QPDEngineTests(object):
                     second = df[second]
                 else:
                     second = Column(second)
-                res = self.op.binary_logical_op(first, second, op).native.tolist()
+                res = self.to_pandas_series(
+                    self.op.binary_logical_op(first, second, op)
+                ).tolist()
                 res = [True if x > 0 else (False if x == 0 else None) for x in res]
                 assert_df_eq(pd.DataFrame({"a": res}), pd.DataFrame({"a": expected}))
 
@@ -102,7 +105,9 @@ class QPDEngineTests(object):
         def test_is_value(self):
             def assert_eq(values, expr, positive, expected):
                 col = self.op.to_col(values)
-                res = self.op.is_value(col, IsValueSpec(expr, positive)).native
+                res = self.to_pandas_series(
+                    self.op.is_value(col, IsValueSpec(expr, positive))
+                )
                 assert expected == res.tolist()
 
             assert_eq([1, 2, None], "null", False, [True, True, False])
@@ -115,8 +120,9 @@ class QPDEngineTests(object):
         def test_is_in(self):
             def assert_eq(df, col, positive, collection, expected):
                 comp = [df[x] if isinstance(x, str) else x for x in collection]
-                res = self.op.is_in(df[col], *comp, positive=positive).native.tolist()
-                assert_df_eq(pd.DataFrame({"a": res}), pd.DataFrame({"a": expected}))
+                self.assert_column_eq(
+                    self.op.is_in(df[col], *comp, positive=positive), expected
+                )
 
             df = self.to_df([[0, 1, 1], [2, 3, 4]], ["a", "b", "c"])
             assert_eq(df, "c", True, [], [False, False])
@@ -140,10 +146,12 @@ class QPDEngineTests(object):
 
         def test_is_between(self):
             def assert_eq(df, col, lower, upper, positive, expected):
-                res = self.op.is_between(
-                    df[col], df[lower], df[upper], positive=positive
-                ).native.tolist()
-                assert_df_eq(pd.DataFrame({"a": res}), pd.DataFrame({"a": expected}))
+                self.assert_column_eq(
+                    self.op.is_between(
+                        df[col], df[lower], df[upper], positive=positive
+                    ),
+                    expected,
+                )
 
             df = self.to_df(
                 [[0, 1, 1], [2, 3, 4], [None, 1, 1], [1, None, 1], [1, 1, None]],
@@ -158,8 +166,7 @@ class QPDEngineTests(object):
                     return df[v] if isinstance(v, str) else Column(v)
 
                 cols = [convert(x) for x in cols]
-                res = self.op.case_when(*cols).native
-                assert expected == res.tolist()
+                self.assert_column_eq(self.op.case_when(*cols), expected)
 
             a = self.to_df(
                 [
@@ -176,7 +183,7 @@ class QPDEngineTests(object):
 
         def test_drop_duplicates(self):
             def assert_eq(df, expected, expected_cols):
-                res = self.op.to_native(self.op.drop_duplicates(df))
+                res = self.to_pandas_df(self.op.drop_duplicates(df))
                 assert_df_eq(res, expected, expected_cols)
 
             a = self.to_df([["x", "a"], ["x", "a"], [None, None]], ["a", "b"])
@@ -184,7 +191,7 @@ class QPDEngineTests(object):
 
         def test_order_by_limit(self):
             def assert_eq(df, order_by, limit, expected, expected_cols):
-                res = self.op.to_native(
+                res = self.to_pandas_df(
                     self.op.order_by_limit(df, OrderBySpec(*order_by), limit)
                 )
                 assert_df_eq(res, expected, expected_cols)
@@ -199,7 +206,7 @@ class QPDEngineTests(object):
 
         def test_union(self):
             def assert_eq(df1, df2, unique, expected, expected_cols):
-                res = self.op.to_native(self.op.union(df1, df2, unique=unique))
+                res = self.to_pandas_df(self.op.union(df1, df2, unique=unique))
                 assert_df_eq(res, expected, expected_cols)
 
             a = self.to_df([["x", "a"], ["x", "a"], [None, None]], ["a", "b"])
@@ -228,7 +235,7 @@ class QPDEngineTests(object):
 
         def test_intersect(self):
             def assert_eq(df1, df2, unique, expected, expected_cols):
-                res = self.op.to_native(self.op.intersect(df1, df2, unique=unique))
+                res = self.to_pandas_df(self.op.intersect(df1, df2, unique=unique))
                 assert_df_eq(res, expected, expected_cols)
 
             a = self.to_df([["x", "a"], ["x", "a"], [None, None]], ["a", "b"])
@@ -243,7 +250,7 @@ class QPDEngineTests(object):
 
         def test_except(self):
             def assert_eq(df1, df2, unique, expected, expected_cols):
-                res = self.op.to_native(self.op.except_df(df1, df2, unique=unique))
+                res = self.to_pandas_df(self.op.except_df(df1, df2, unique=unique))
                 assert_df_eq(res, expected, expected_cols)
 
             a = self.to_df([["x", "a"], ["x", "a"], [None, None]], ["a", "b"])
@@ -256,7 +263,7 @@ class QPDEngineTests(object):
 
         def test_joins(self):
             def assert_eq(df1, df2, join_type, on, expected, expected_cols):
-                res = self.op.to_native(
+                res = self.to_pandas_df(
                     self.op.join(df1, df2, join_type=join_type, on=on)
                 )
                 assert_df_eq(res, expected, expected_cols)
@@ -347,14 +354,15 @@ class QPDEngineTests(object):
                 return WindowFunctionSpec("row_number", False, False, ws)
 
             def assert_eq(df, index, *order_by):
-                e = df[df.columns]
-                e["x"] = index
+                ndf = self.op.to_native(df)
+                e = ndf[ndf.columns]
+                e["x"] = self.op.to_col(index).native
                 self.w_eq(
                     df, ob(*order_by), [], "x", e.values.tolist(), list(e.columns)
                 )
 
-            df = self.to_native_df(
-                [[0, 10, None], [1, None, 100], [None, 20, 300]], columns=list("abc")
+            df = self.to_df(
+                [[0, 10, None], [1, None, 100], [None, 20, 300]], list("abc")
             )
             assert_eq(df, [1, 2, 3])
             assert_eq(df, [2, 3, 1], ("a", True))
@@ -364,9 +372,9 @@ class QPDEngineTests(object):
             assert_eq(df, [3, 2, 1], ("a", False, "first"))
             assert_eq(df, [2, 1, 3], ("a", False, "last"))
 
-            df = self.to_native_df(
+            df = self.to_df(
                 [[0, 10, None], [0, 5, None], [0, None, 100], [None, 20, 300]],
-                columns=list("abc"),
+                list("abc"),
             )
             assert_eq(df, [4, 3, 2, 1], ("a", True), ("b", True))
             assert_eq(df, [2, 3, 4, 1], ("a", True), ("b", False))
@@ -374,17 +382,12 @@ class QPDEngineTests(object):
             assert_eq(df, [3, 4, 2, 1], ("a", True), ("b", False, "first"))
             assert_eq(df, [2, 3, 1, 4], ("a", True, "last"), ("b", False, "first"))
 
-        def test_agg(self):
-            def assert_eq(pdf, gp_keys, gp_map, expected, expected_cols):
-                op = self.op
-                df = op.to_df(pdf)
-                res = op.to_native(op.group_agg(df, gp_keys, gp_map))
+        def test_agg_first_last(self):
+            def assert_eq(df, gp_keys, gp_map, expected, expected_cols):
+                res = self.to_pandas_df(self.op.group_agg(df, gp_keys, gp_map))
                 assert_df_eq(res, expected, expected_cols)
 
-            nan = float("nan")
-            pdf = self.to_native_df(
-                [[0, 1], [0, 1], [1, None], [1, 5], [0, None]], columns=["a", "b"]
-            )
+            pdf = self.to_df([[0, 1], [0, 1], [1, None], [1, 5], [0, None]], ["a", "b"])
             assert_eq(
                 pdf,
                 ["a"],
@@ -394,12 +397,16 @@ class QPDEngineTests(object):
                     "b3": ("b", AggFunctionSpec("last", unique=False, dropna=False)),
                     "b4": ("b", AggFunctionSpec("last", unique=False, dropna=True)),
                 },
-                [[1.0, 1.0, nan, 1.0], [nan, 5.0, 5.0, 5.0]],
+                [[1.0, 1.0, None, 1.0], [None, 5.0, 5.0, 5.0]],
                 ["b1", "b2", "b3", "b4"],
             )
-            pdf = self.to_native_df(
-                [[0, 1], [0, 1], [0, 2], [1, 5], [0, None]], columns=["a", "b"]
-            )
+
+        def test_agg_sum_mean(self):
+            def assert_eq(df, gp_keys, gp_map, expected, expected_cols):
+                res = self.to_pandas_df(self.op.group_agg(df, gp_keys, gp_map))
+                assert_df_eq(res, expected, expected_cols)
+
+            pdf = self.to_df([[0, 1], [0, 1], [0, 2], [1, 5], [0, None]], ["a", "b"])
             assert_eq(
                 pdf,
                 ["a"],
@@ -412,36 +419,7 @@ class QPDEngineTests(object):
                 [[4, 3, 3, 4], [5, 5, 5, 5]],
                 ["b1", "b2", "b3", "b4"],
             )
-            pdf = self.to_native_df(
-                [[0, 1], [0, 1], [0, 2], [1, None], [0, None]], columns=["a", "b"]
-            )
-            assert_eq(
-                pdf,
-                ["a"],
-                {
-                    "b1": ("b", AggFunctionSpec("min", unique=False, dropna=False)),
-                    "b2": ("b", AggFunctionSpec("min", unique=True, dropna=False)),
-                    "b3": ("b", AggFunctionSpec("min", unique=True, dropna=True)),
-                    "b4": ("b", AggFunctionSpec("min", unique=False, dropna=True)),
-                },
-                [[1.0, 1.0, 1.0, 1.0], [nan, nan, nan, nan]],
-                ["b1", "b2", "b3", "b4"],
-            )
-            assert_eq(
-                pdf,
-                ["a"],
-                {
-                    "b1": ("b", AggFunctionSpec("max", unique=False, dropna=False)),
-                    "b2": ("b", AggFunctionSpec("max", unique=True, dropna=False)),
-                    "b3": ("b", AggFunctionSpec("max", unique=True, dropna=True)),
-                    "b4": ("b", AggFunctionSpec("max", unique=False, dropna=True)),
-                },
-                [[2.0, 2.0, 2.0, 2.0], [nan, nan, nan, nan]],
-                ["b1", "b2", "b3", "b4"],
-            )
-            pdf = self.to_native_df(
-                [[0, 1], [0, 1], [0, 4], [1, 5], [0, None]], columns=["a", "b"]
-            )
+            pdf = self.to_df([[0, 1], [0, 1], [0, 4], [1, 5], [0, None]], ["a", "b"])
             assert_eq(
                 pdf,
                 ["a"],
@@ -454,9 +432,82 @@ class QPDEngineTests(object):
                 [[2, 2.5, 2.5, 2], [5, 5, 5, 5]],
                 ["b1", "b2", "b3", "b4"],
             )
-            pdf = self.to_native_df(
-                [[0, 1], [0, 1], [0, 4], [1, None], [0, None]], columns=["a", "b"]
+            assert_eq(
+                pdf,
+                ["a"],
+                {
+                    "b1": ("b", AggFunctionSpec("sum", unique=True, dropna=False)),
+                    "b2": ("b", AggFunctionSpec("avg", unique=True, dropna=False)),
+                },
+                [[5, 2.5], [5, 5]],
+                ["b1", "b2"],
             )
+
+        def test_agg_min_max(self):
+            def assert_eq(df, gp_keys, gp_map, expected, expected_cols):
+                res = self.to_pandas_df(self.op.group_agg(df, gp_keys, gp_map))
+                assert_df_eq(res, expected, expected_cols)
+
+            pdf = self.to_df([[0, 1], [0, 1], [0, 2], [1, None], [0, None]], ["a", "b"])
+            assert_eq(
+                pdf,
+                ["a"],
+                {
+                    "b1": ("b", AggFunctionSpec("min", unique=False, dropna=False)),
+                    "b2": ("b", AggFunctionSpec("min", unique=True, dropna=False)),
+                    "b3": ("b", AggFunctionSpec("min", unique=True, dropna=True)),
+                    "b4": ("b", AggFunctionSpec("min", unique=False, dropna=True)),
+                },
+                [[1.0, 1.0, 1.0, 1.0], [None, None, None, None]],
+                ["b1", "b2", "b3", "b4"],
+            )
+            assert_eq(
+                pdf,
+                ["a"],
+                {
+                    "b1": ("b", AggFunctionSpec("max", unique=False, dropna=False)),
+                    "b2": ("b", AggFunctionSpec("max", unique=True, dropna=False)),
+                    "b3": ("b", AggFunctionSpec("max", unique=True, dropna=True)),
+                    "b4": ("b", AggFunctionSpec("max", unique=False, dropna=True)),
+                },
+                [[2.0, 2.0, 2.0, 2.0], [None, None, None, None]],
+                ["b1", "b2", "b3", "b4"],
+            )
+
+            pdf = self.to_df(
+                [[0, "a"], [0, "a"], [0, "b"], [1, None], [0, None]], ["a", "b"]
+            )
+            assert_eq(
+                pdf,
+                ["a"],
+                {
+                    "b1": ("b", AggFunctionSpec("min", unique=False, dropna=False)),
+                    "b2": ("b", AggFunctionSpec("min", unique=True, dropna=False)),
+                    "b3": ("b", AggFunctionSpec("min", unique=True, dropna=True)),
+                    "b4": ("b", AggFunctionSpec("min", unique=False, dropna=True)),
+                },
+                [["a", "a", "a", "a"], [None, None, None, None]],
+                ["b1", "b2", "b3", "b4"],
+            )
+            assert_eq(
+                pdf,
+                ["a"],
+                {
+                    "b1": ("b", AggFunctionSpec("max", unique=False, dropna=False)),
+                    "b2": ("b", AggFunctionSpec("max", unique=True, dropna=False)),
+                    "b3": ("b", AggFunctionSpec("max", unique=True, dropna=True)),
+                    "b4": ("b", AggFunctionSpec("max", unique=False, dropna=True)),
+                },
+                [["b", "b", "b", "b"], [None, None, None, None]],
+                ["b1", "b2", "b3", "b4"],
+            )
+
+        def test_agg_count(self):
+            def assert_eq(df, gp_keys, gp_map, expected, expected_cols):
+                res = self.to_pandas_df(self.op.group_agg(df, gp_keys, gp_map))
+                assert_df_eq(res, expected, expected_cols)
+
+            pdf = self.to_df([[0, 1], [0, 1], [0, 4], [1, None], [0, None]], ["a", "b"])
             # count on single column
             assert_eq(
                 pdf,
@@ -493,7 +544,7 @@ class QPDEngineTests(object):
                 ["a1", "a2"],
             )
             # count multiple cols
-            pdf = self.to_native_df(
+            pdf = self.to_df(
                 [
                     [0, 1, None],
                     [0, 1, None],
@@ -501,7 +552,7 @@ class QPDEngineTests(object):
                     [1, None, None],
                     [None, None, None],
                 ],
-                columns=["a", "b", "c"],
+                ["a", "b", "c"],
             )
             assert_eq(
                 pdf,
@@ -514,8 +565,13 @@ class QPDEngineTests(object):
                 ["a1", "a2"],
             )
 
+        def test_agg_combined(self):
+            def assert_eq(df, gp_keys, gp_map, expected, expected_cols):
+                res = self.to_pandas_df(self.op.group_agg(df, gp_keys, gp_map))
+                assert_df_eq(res, expected, expected_cols)
+
             # combined case
-            pdf = self.to_native_df(
+            pdf = self.to_df(
                 [
                     [0, 1, None],
                     [0, 1, None],
@@ -523,38 +579,33 @@ class QPDEngineTests(object):
                     [1, None, None],
                     [None, None, None],
                 ],
-                columns=["a", "b", "c"],
+                ["a", "b", "c"],
             )
             assert_eq(
                 pdf,
                 ["a"],
                 {
-                    "a1": ("a", AggFunctionSpec("first", unique=False, dropna=False)),
+                    "a1": ("a", AggFunctionSpec("sum", unique=False, dropna=False)),
                     "a2": ("*", AggFunctionSpec("count", unique=True, dropna=False)),
-                    "a3": ("c", AggFunctionSpec("first", unique=True, dropna=False)),
+                    "a3": ("c", AggFunctionSpec("sum", unique=True, dropna=False)),
                     "a4": ("*", AggFunctionSpec("count", unique=False, dropna=False)),
                     "a5": ("c", AggFunctionSpec("min", unique=False, dropna=False)),
                     "a6": ("b,c", AggFunctionSpec("count", unique=True, dropna=False)),
                 },
                 [
-                    [0.0, 2.0, nan, 3.0, 1.0, 2.0],
-                    [1.0, 1.0, nan, 1.0, nan, 1.0],
-                    [nan, 1.0, nan, 1.0, nan, 1.0],
+                    [0.0, 2.0, 1, 3.0, 1.0, 2.0],
+                    [1.0, 1.0, None, 1.0, None, 1.0],
+                    [None, 1.0, None, 1.0, None, 1.0],
                 ],
                 ["a1", "a2", "a3", "a4", "a5", "a6"],
             )
 
-        def test_agg_no_group(self):
-            def assert_eq(pdf, gp_map, expected, expected_cols):
-                op = self.op
-                df = op.to_df(pdf)
-                res = op.to_native(op.group_agg(df, [], gp_map))
+        def test_agg_no_group_first_last(self):
+            def assert_eq(df, gp_map, expected, expected_cols):
+                res = self.to_pandas_df(self.op.group_agg(df, [], gp_map))
                 assert_df_eq(res, expected, expected_cols)
 
-            nan = float("nan")
-            pdf = self.to_native_df(
-                [[0, 1], [0, 1], [1, None], [1, 5], [0, None]], columns=["a", "b"]
-            )
+            pdf = self.to_df([[0, 1], [0, 1], [1, None], [1, 5], [0, None]], ["a", "b"])
             assert_eq(
                 pdf,
                 {
@@ -563,9 +614,16 @@ class QPDEngineTests(object):
                     "b3": ("b", AggFunctionSpec("last", unique=False, dropna=False)),
                     "b4": ("b", AggFunctionSpec("last", unique=False, dropna=True)),
                 },
-                [[1.0, 1.0, nan, 5.0]],
+                [[1.0, 1.0, None, 5.0]],
                 ["b1", "b2", "b3", "b4"],
             )
+
+        def test_agg_no_group_sum(self):
+            def assert_eq(df, gp_map, expected, expected_cols):
+                res = self.to_pandas_df(self.op.group_agg(df, [], gp_map))
+                assert_df_eq(res, expected, expected_cols)
+
+            pdf = self.to_df([[0, 1], [0, 1], [1, None], [1, 5], [0, None]], ["a", "b"])
             assert_eq(
                 pdf,
                 {
@@ -577,6 +635,13 @@ class QPDEngineTests(object):
                 [[7, 6, 6, 7]],
                 ["b1", "b2", "b3", "b4"],
             )
+
+        def test_agg_no_group_min_max(self):
+            def assert_eq(df, gp_map, expected, expected_cols):
+                res = self.to_pandas_df(self.op.group_agg(df, [], gp_map))
+                assert_df_eq(res, expected, expected_cols)
+
+            pdf = self.to_df([[0, 1], [0, 1], [1, None], [1, 5], [0, None]], ["a", "b"])
             assert_eq(
                 pdf,
                 {
@@ -599,9 +664,13 @@ class QPDEngineTests(object):
                 [[5, 5, 5, 5]],
                 ["b1", "b2", "b3", "b4"],
             )
-            pdf = self.to_native_df(
-                [[0, 1], [0, 1], [0, 4], [0, None]], columns=["a", "b"]
-            )
+
+        def test_agg_no_group_mean(self):
+            def assert_eq(df, gp_map, expected, expected_cols):
+                res = self.to_pandas_df(self.op.group_agg(df, [], gp_map))
+                assert_df_eq(res, expected, expected_cols)
+
+            pdf = self.to_df([[0, 1], [0, 1], [0, 4], [0, None]], ["a", "b"])
             assert_eq(
                 pdf,
                 {
@@ -613,7 +682,13 @@ class QPDEngineTests(object):
                 [[2, 2.5, 2.5, 2]],
                 ["b1", "b2", "b3", "b4"],
             )
-            pdf = self.to_native_df([[0, 1], [0, 1], [1, None]], columns=["a", "b"])
+
+        def test_agg_no_group_count(self):
+            def assert_eq(df, gp_map, expected, expected_cols):
+                res = self.to_pandas_df(self.op.group_agg(df, [], gp_map))
+                assert_df_eq(res, expected, expected_cols)
+
+            pdf = self.to_df([[0, 1], [0, 1], [1, None]], ["a", "b"])
             # count on single column
             assert_eq(
                 pdf,
@@ -637,7 +712,7 @@ class QPDEngineTests(object):
                 ["b1", "b2"],
             )
             # count multiple cols
-            pdf = self.to_native_df(
+            pdf = self.to_df(
                 [
                     [0, 1, None],
                     [0, 1, None],
@@ -645,7 +720,7 @@ class QPDEngineTests(object):
                     [1, None, None],
                     [None, None, None],
                 ],
-                columns=["a", "b", "c"],
+                ["a", "b", "c"],
             )
             assert_eq(
                 pdf,
@@ -656,6 +731,22 @@ class QPDEngineTests(object):
                 [[5, 3]],
                 ["a1", "a2"],
             )
+
+        def test_agg_no_group_combined(self):
+            def assert_eq(df, gp_map, expected, expected_cols):
+                res = self.to_pandas_df(self.op.group_agg(df, [], gp_map))
+                assert_df_eq(res, expected, expected_cols)
+
+            pdf = self.to_df(
+                [
+                    [0, 1, None],
+                    [0, 1, None],
+                    [0, 4, 1],
+                    [1, None, None],
+                    [None, None, None],
+                ],
+                ["a", "b", "c"],
+            )
             # combo test
             assert_eq(
                 pdf,
@@ -679,7 +770,7 @@ class QPDEngineTests(object):
                 )
                 return WindowFunctionSpec("row_number", False, False, ws)
 
-            a = self.to_native_df([[0, 1], [0, 2]], columns=["a", "b"])
+            a = self.to_df([[0, 1], [0, 2]], ["a", "b"])
             self.w_eq(
                 a, make_func([], []), [], "x", [[0, 1, 1], [0, 2, 2]], ["a", "b", "x"]
             )
@@ -691,9 +782,7 @@ class QPDEngineTests(object):
                 [[0, 1, 2], [0, 2, 1]],
                 ["a", "b", "x"],
             )
-            a = self.to_native_df(
-                [[0, 1], [0, None], [None, 1], [None, None]], columns=["a", "b"]
-            )
+            a = self.to_df([[0, 1], [0, None], [None, 1], [None, None]], ["a", "b"])
             self.w_eq(
                 a,
                 make_func([], [("a", True), ("b", True)]),
@@ -710,16 +799,7 @@ class QPDEngineTests(object):
                 [[0, 1, 1], [0, None, 2], [None, 1, 3], [None, None, 4]],
                 ["a", "b", "x"],
             )
-            self.w_eq(
-                a,
-                make_func(["a"], [("a", True), ("b", True)]),
-                [],
-                "x",
-                [[0, 1, 2], [0, None, 1], [None, 1, 2], [None, None, 1]],
-                ["a", "b", "x"],
-            )
-
-            a = self.to_native_df([[0, 1], [0, 4], [2, 3]], columns=["a", "b"])
+            a = self.to_df([[0, 1], [0, 4], [2, 3]], ["a", "b"])
             self.w_eq(
                 a,
                 make_func([], [("b", False)]),
@@ -729,6 +809,26 @@ class QPDEngineTests(object):
                 ["a", "b", "x"],
             )
 
+        def test_window_row_number_partition_by(self):
+            def make_func(partition_keys, order_by):
+                ws = WindowSpec(
+                    "",
+                    partition_keys=partition_keys,
+                    order_by=OrderBySpec(*order_by),
+                    windowframe=make_windowframe_spec(""),
+                )
+                return WindowFunctionSpec("row_number", False, False, ws)
+
+            a = self.to_df([[0, 1], [0, None], [None, 1], [None, None]], ["a", "b"])
+            self.w_eq(
+                a,
+                make_func(["a"], [("a", True), ("b", True)]),
+                [],
+                "x",
+                [[0, 1, 2], [0, None, 1], [None, 1, 2], [None, None, 1]],
+                ["a", "b", "x"],
+            )
+
         def test_window_ranks(self):
             def make_func(func, partition_keys, order_by):
                 ws = WindowSpec(
@@ -739,9 +839,7 @@ class QPDEngineTests(object):
                 )
                 return WindowFunctionSpec(func, False, False, ws)
 
-            a = self.to_native_df(
-                [[0, 1], [0, 1], [None, 1], [0, None]], columns=["a", "b"]
-            )
+            a = self.to_df([[0, 1], [0, 1], [None, 1], [0, None]], ["a", "b"])
             self.w_eq(
                 a,
                 make_func("rank", [], ["b"]),
@@ -760,24 +858,6 @@ class QPDEngineTests(object):
                 ["a", "b", "x"],
             )
 
-            self.w_eq(
-                a,
-                make_func("rank", ["a"], ["b"]),
-                [],
-                "x",
-                [[0, 1, 2], [0, 1, 2], [None, 1, 1], [0, None, 1]],
-                ["a", "b", "x"],
-            )
-
-            self.w_eq(
-                a,
-                make_func("rank", ["a"], [("b", False)]),
-                [],
-                "x",
-                [[0, 1, 1], [0, 1, 1], [None, 1, 1], [0, None, 3]],
-                ["a", "b", "x"],
-            )
-
             self.w_eq(
                 a,
                 make_func("dense_rank", [], ["b"]),
@@ -796,24 +876,6 @@ class QPDEngineTests(object):
                 ["a", "b", "x"],
             )
 
-            self.w_eq(
-                a,
-                make_func("dense_rank", ["a"], ["b"]),
-                [],
-                "x",
-                [[0, 1, 2], [0, 1, 2], [None, 1, 1], [0, None, 1]],
-                ["a", "b", "x"],
-            )
-
-            self.w_eq(
-                a,
-                make_func("dense_rank", ["a"], [("b", False)]),
-                [],
-                "x",
-                [[0, 1, 1], [0, 1, 1], [None, 1, 1], [0, None, 2]],
-                ["a", "b", "x"],
-            )
-
             self.w_eq(
                 a,
                 make_func("percentile_rank", [], ["b"]),
@@ -837,6 +899,53 @@ class QPDEngineTests(object):
                 ["a", "b", "x"],
             )
 
+        def test_window_ranks_partition_by(self):
+            def make_func(func, partition_keys, order_by):
+                ws = WindowSpec(
+                    "",
+                    partition_keys=partition_keys,
+                    order_by=OrderBySpec(*order_by),
+                    windowframe=make_windowframe_spec(""),
+                )
+                return WindowFunctionSpec(func, False, False, ws)
+
+            a = self.to_df([[0, 1], [0, 1], [None, 1], [0, None]], ["a", "b"])
+            self.w_eq(
+                a,
+                make_func("rank", ["a"], ["b"]),
+                [],
+                "x",
+                [[0, 1, 2], [0, 1, 2], [None, 1, 1], [0, None, 1]],
+                ["a", "b", "x"],
+            )
+
+            self.w_eq(
+                a,
+                make_func("rank", ["a"], [("b", False)]),
+                [],
+                "x",
+                [[0, 1, 1], [0, 1, 1], [None, 1, 1], [0, None, 3]],
+                ["a", "b", "x"],
+            )
+
+            self.w_eq(
+                a,
+                make_func("dense_rank", ["a"], ["b"]),
+                [],
+                "x",
+                [[0, 1, 2], [0, 1, 2], [None, 1, 1], [0, None, 1]],
+                ["a", "b", "x"],
+            )
+
+            self.w_eq(
+                a,
+                make_func("dense_rank", ["a"], [("b", False)]),
+                [],
+                "x",
+                [[0, 1, 1], [0, 1, 1], [None, 1, 1], [0, None, 2]],
+                ["a", "b", "x"],
+            )
+
             self.w_eq(
                 a,
                 make_func("percentile_rank", ["a"], ["b"]),
@@ -865,9 +974,7 @@ class QPDEngineTests(object):
                 )
                 return WindowFunctionSpec(func, False, False, ws)
 
-            a = self.to_native_df(
-                [[0, 1], [0, 1], [None, 1], [0, None]], columns=["a", "b"]
-            )
+            a = self.to_df([[0, 1], [0, 1], [None, 1], [0, None]], ["a", "b"])
             self.w_eq(
                 a,
                 make_func("lag", [], []),
@@ -884,14 +991,6 @@ class QPDEngineTests(object):
                 [[0, 1, 100], [0, 1, 1], [None, 1, 1], [0, None, 1]],
                 ["a", "b", "x"],
             )
-            self.w_eq(
-                a,
-                make_func("lag", ["a"], ["b"]),
-                [col("b"), 1],
-                "x",
-                [[0, 1, None], [0, 1, 1], [None, 1, None], [0, None, None]],
-                ["a", "b", "x"],
-            )
 
             self.w_eq(
                 a,
@@ -909,6 +1008,27 @@ class QPDEngineTests(object):
                 [[0, 1, 1], [0, 1, 1], [None, 1, None], [0, None, 100]],
                 ["a", "b", "x"],
             )
+
+        def test_window_lead_lag_partition_by(self):
+            def make_func(func, partition_keys, order_by):
+                ws = WindowSpec(
+                    "",
+                    partition_keys=partition_keys,
+                    order_by=OrderBySpec(*order_by),
+                    windowframe=make_windowframe_spec(""),
+                )
+                return WindowFunctionSpec(func, False, False, ws)
+
+            a = self.to_df([[0, 1], [0, 1], [None, 1], [0, None]], ["a", "b"])
+            self.w_eq(
+                a,
+                make_func("lag", ["a"], ["b"]),
+                [col("b"), 1],
+                "x",
+                [[0, 1, None], [0, 1, 1], [None, 1, None], [0, None, None]],
+                ["a", "b", "x"],
+            )
+
             self.w_eq(
                 a,
                 make_func("lead", ["a"], ["b"]),
@@ -929,8 +1049,8 @@ class QPDEngineTests(object):
                 )
                 return WindowFunctionSpec("sum", False, False, ws)
 
+            a = self.to_df([[0, 1], [0, 2]], ["a", "b"])
             # no partition no window frame
-            a = self.to_native_df([[0, 1], [0, 2]], columns=["a", "b"])
             self.w_eq(
                 a,
                 make_func([], []),
@@ -948,19 +1068,7 @@ class QPDEngineTests(object):
                 ["a", "b", "x"],
             )
 
-            # partition no window frame
-            a = self.to_native_df(
-                [[0, 1], [None, 1], [0, None], [0, 2]], columns=["a", "b"]
-            )
-            self.w_eq(
-                a,
-                make_func(["a"], []),
-                [col("b")],
-                "x",
-                [[0, 1, 3], [None, 1, 1], [0, None, 3], [0, 2, 3]],
-                ["a", "b", "x"],
-            )
-
+            a = self.to_df([[0, 1], [None, 1], [0, None], [0, 2]], ["a", "b"])
             # expanding, no partition
             self.w_eq(
                 a,
@@ -971,16 +1079,6 @@ class QPDEngineTests(object):
                 ["a", "b", "x"],
             )
 
-            # expanding, with partition
-            self.w_eq(
-                a,
-                make_func(["a"], ["b", "a"], end=0),
-                [col("b")],
-                "x",
-                [[0, 1, 1], [None, 1, 1], [0, None, None], [0, 2, 3]],
-                ["a", "b", "x"],
-            )
-
             # rolling, no partition
             self.w_eq(
                 a,
@@ -991,6 +1089,70 @@ class QPDEngineTests(object):
                 ["a", "b", "x"],
             )
 
+            if not pd.__version__.startswith("1.1"):
+                # irregular, no partition
+                self.w_eq(
+                    a,
+                    make_func([], ["b", "a"], end=-1),
+                    [col("b")],
+                    "x",
+                    [[0, 1, 1], [None, 1, None], [0, None, None], [0, 2, 2]],
+                    ["a", "b", "x"],
+                )
+
+                # irregular, no partition
+                self.w_eq(
+                    a,
+                    make_func([], ["b", "a"], start=-2, end=-1),
+                    [col("b")],
+                    "x",
+                    [[0, 1, 1], [None, 1, None], [0, None, None], [0, 2, 2]],
+                    ["a", "b", "x"],
+                )
+
+                # irregular, no partition
+                self.w_eq(
+                    a,
+                    make_func([], ["b", "a"], start=1, end=None),
+                    [col("b")],
+                    "x",
+                    [[0, 1, 2], [None, 1, 3], [0, None, 4], [0, 2, None]],
+                    ["a", "b", "x"],
+                )
+
+        def test_window_sum_partition_by(self):
+            def make_func(partition_keys, order_by, start=None, end=None):
+                wtype = "" if start is None and end is None else "rows"
+                ws = WindowSpec(
+                    "",
+                    partition_keys=partition_keys,
+                    order_by=OrderBySpec(*order_by),
+                    windowframe=make_windowframe_spec(wtype, start, end),
+                )
+                return WindowFunctionSpec("sum", False, False, ws)
+
+            # no partition no window frame
+            # partition no window frame
+            a = self.to_df([[0, 1], [None, 1], [0, None], [0, 2]], ["a", "b"])
+            self.w_eq(
+                a,
+                make_func(["a"], []),
+                [col("b")],
+                "x",
+                [[0, 1, 3], [None, 1, 1], [0, None, 3], [0, 2, 3]],
+                ["a", "b", "x"],
+            )
+
+            # expanding, with partition
+            self.w_eq(
+                a,
+                make_func(["a"], ["b", "a"], end=0),
+                [col("b")],
+                "x",
+                [[0, 1, 1], [None, 1, 1], [0, None, None], [0, 2, 3]],
+                ["a", "b", "x"],
+            )
+
             # rolling, with partition
             self.w_eq(
                 a,
@@ -1009,7 +1171,7 @@ class QPDEngineTests(object):
                 ["a", "b", "x"],
             )
 
-            if not pd.__version__.startswith("1.1.0"):
+            if not pd.__version__.startswith("1.1"):
                 # irregular, with partition
                 self.w_eq(
                     a,
@@ -1020,16 +1182,6 @@ class QPDEngineTests(object):
                     ["a", "b", "x"],
                 )
 
-                # irregular, no partition
-                self.w_eq(
-                    a,
-                    make_func([], ["b", "a"], end=-1),
-                    [col("b")],
-                    "x",
-                    [[0, 1, 1], [None, 1, None], [0, None, None], [0, 2, 2]],
-                    ["a", "b", "x"],
-                )
-
                 # irregular, with partition
                 self.w_eq(
                     a,
@@ -1040,16 +1192,6 @@ class QPDEngineTests(object):
                     ["a", "b", "x"],
                 )
 
-                # irregular, no partition
-                self.w_eq(
-                    a,
-                    make_func([], ["b", "a"], start=-2, end=-1),
-                    [col("b")],
-                    "x",
-                    [[0, 1, 1], [None, 1, None], [0, None, None], [0, 2, 2]],
-                    ["a", "b", "x"],
-                )
-
                 # irregular, with partition
                 self.w_eq(
                     a,
@@ -1060,17 +1202,7 @@ class QPDEngineTests(object):
                     ["a", "b", "x"],
                 )
 
-                # irregular, no partition
-                self.w_eq(
-                    a,
-                    make_func([], ["b", "a"], start=1, end=None),
-                    [col("b")],
-                    "x",
-                    [[0, 1, 2], [None, 1, 3], [0, None, 4], [0, 2, None]],
-                    ["a", "b", "x"],
-                )
-
-        def test_window_simple_agg(self):
+        def test_window_simple_agg_partition_by(self):
             def make_func(func, partition_keys, order_by, start=None, end=None):
                 wtype = "" if start is None and end is None else "rows"
                 ws = WindowSpec(
@@ -1081,9 +1213,7 @@ class QPDEngineTests(object):
                 )
                 return WindowFunctionSpec(func, False, False, ws)
 
-            a = self.to_native_df(
-                [[0, 1], [None, 1], [0, None], [0, 2]], columns=["a", "b"]
-            )
+            a = self.to_df([[0, 1], [None, 1], [0, None], [0, 2]], ["a", "b"])
             self.w_eq(
                 a,
                 make_func("max", ["a"], ["b"]),
@@ -1143,9 +1273,7 @@ class QPDEngineTests(object):
                 [[0, 1, 2], [None, 1, 1], [0, None, 2], [0, 2, 2]],
                 ["a", "b", "x"],
             )
-            a = self.to_native_df(
-                [[0, 1], [None, 1], [0, None], [0, 3]], columns=["a", "b"]
-            )
+            a = self.to_df([[0, 1], [None, 1], [0, None], [0, 3]], ["a", "b"])
             self.w_eq(
                 a,
                 make_func("count", ["a"], [("b", True)], start=-1, end=0),
@@ -1155,13 +1283,12 @@ class QPDEngineTests(object):
                 ["a", "b", "x"],
             )
 
-        def w_eq(self, pdf, func, cols, dest, expected, expected_cols):
-            df = self.op.to_df(pdf)
+        def w_eq(self, df, func, cols, dest, expected, expected_cols):
             cols = [
                 x if isinstance(x, ArgumentSpec) else ArgumentSpec(False, x)
                 for x in cols
             ]
-            res = self.op.to_native(self.op.window(df, func, cols, dest))
+            res = self.to_pandas_df(self.op.window(df, func, cols, dest))
             assert_df_eq(res, expected, expected_cols)
 
 
