@@ -5,10 +5,12 @@ import pandas as pd
 class SQLTests(object):
     class Tests(TestsBase):
         def test_basic_select_from(self):
-            df = self.make_rand_df(100, a=(int, 2), b=(str, 3), c=(float, 4))
+            df = self.make_rand_df(5, a=(int, 2), b=(str, 3), c=(float, 4))
             self.eq_sqlite("SELECT * FROM a", a=df)
             self.eq_sqlite("SELECT * FROM a AS x", a=df)
-            self.eq_sqlite("SELECT b AS bb, (a+1) AS cc, x.* FROM a AS x", a=df)
+            self.eq_sqlite("SELECT b AS bb, a+1-2*3.0/4 AS cc, x.* FROM a AS x", a=df)
+            self.eq_sqlite("SELECT *, 1 AS x, 2.5 AS y, 'z' AS z FROM a AS x", a=df)
+            self.eq_sqlite("SELECT *, -(1.0+a)/3 AS x, +(2.5) AS y FROM a AS x", a=df)
 
         def test_case_when(self):
             a = self.make_rand_df(100, a=(int, 20), b=(str, 30), c=(float, 40))
@@ -26,6 +28,23 @@ class SQLTests(object):
             )
 
         def test_drop_duplicates(self):
+            # simplest
+            a = self.make_rand_df(100, a=int, b=int)
+            self.eq_sqlite(
+                """
+                SELECT DISTINCT b, a FROM a
+                """,
+                a=a,
+            )
+            # mix of number and nan
+            a = self.make_rand_df(100, a=(int, 50), b=(int, 50))
+            self.eq_sqlite(
+                """
+                SELECT DISTINCT b, a FROM a
+                """,
+                a=a,
+            )
+            # mix of number and string and nulls
             a = self.make_rand_df(100, a=(int, 50), b=(str, 50), c=float)
             self.eq_sqlite(
                 """
@@ -34,7 +53,7 @@ class SQLTests(object):
                 a=a,
             )
 
-        def test_order_by_limit(self):
+        def test_order_by_no_limit(self):
             a = self.make_rand_df(100, a=(int, 50), b=(str, 50), c=float)
             self.eq_sqlite(
                 """
@@ -42,6 +61,15 @@ class SQLTests(object):
                 """,
                 a=a,
             )
+
+        def test_order_by_limit(self):
+            a = self.make_rand_df(100, a=(int, 50), b=(str, 50), c=float)
+            self.eq_sqlite(
+                """
+                SELECT DISTINCT b, a FROM a LIMIT 0
+                """,
+                a=a,
+            )
             self.eq_sqlite(
                 """
                 SELECT DISTINCT b, a FROM a ORDER BY a LIMIT 2
@@ -58,6 +86,11 @@ class SQLTests(object):
 
         def test_where(self):
             df = self.make_rand_df(100, a=(int, 30), b=(str, 30), c=(float, 30))
+            self.eq_sqlite("SELECT * FROM a WHERE TRUE OR TRUE", a=df)
+            self.eq_sqlite("SELECT * FROM a WHERE TRUE AND TRUE", a=df)
+            self.eq_sqlite("SELECT * FROM a WHERE FALSE OR FALSE", a=df)
+            self.eq_sqlite("SELECT * FROM a WHERE FALSE AND FALSE", a=df)
+
             self.eq_sqlite("SELECT * FROM a WHERE TRUE OR b<='ssssss8'", a=df)
             self.eq_sqlite("SELECT * FROM a WHERE TRUE AND b<='ssssss8'", a=df)
             self.eq_sqlite("SELECT * FROM a WHERE FALSE OR b<='ssssss8'", a=df)
@@ -245,6 +278,15 @@ class SQLTests(object):
             )
 
         def test_agg_sum_avg_no_group_by(self):
+            self.eq_sqlite(
+                """
+                SELECT
+                    SUM(a) AS sum_a,
+                    AVG(a) AS avg_a
+                FROM a
+                """,
+                a=([[float("nan")]], ["a"]),
+            )
             a = self.make_rand_df(
                 100, a=(int, 50), b=(str, 50), c=(int, 30), d=(str, 40), e=(float, 40)
             )
@@ -393,6 +435,30 @@ class SQLTests(object):
                 a=a,
             )
 
+        def test_window_row_number_partition_by(self):
+            a = self.make_rand_df(100, a=int, b=(float, 50))
+            self.eq_sqlite(
+                """
+                SELECT *,
+                    ROW_NUMBER() OVER (PARTITION BY a ORDER BY a,b DESC) AS a5
+                FROM a
+                """,
+                a=a,
+            )
+
+            a = self.make_rand_df(
+                100, a=(int, 50), b=(str, 50), c=(int, 30), d=(str, 40), e=float
+            )
+            self.eq_sqlite(
+                """
+                SELECT *,
+                    ROW_NUMBER() OVER (PARTITION BY a ORDER BY a,b DESC, e) AS a3,
+                    ROW_NUMBER() OVER (PARTITION BY a,c ORDER BY a,b DESC, e) AS a4
+                FROM a
+                """,
+                a=a,
+            )
+
         def test_window_ranks(self):
             a = self.make_rand_df(100, a=int, b=(float, 50), c=(str, 50))
             self.eq_sqlite(
@@ -406,6 +472,22 @@ class SQLTests(object):
                 a=a,
             )
 
+        def test_window_ranks_partition_by(self):
+            a = self.make_rand_df(100, a=int, b=(float, 50), c=(str, 50))
+            self.eq_sqlite(
+                """
+                SELECT *,
+                    RANK() OVER (PARTITION BY a ORDER BY b DESC NULLS FIRST, c) AS a1,
+                    DENSE_RANK() OVER
+                        (PARTITION BY a ORDER BY a ASC, b DESC NULLS LAST, c DESC)
+                        AS a2,
+                    PERCENT_RANK() OVER
+                        (PARTITION BY a ORDER BY a ASC, b ASC NULLS LAST, c) AS a4
+                FROM a
+                """,
+                a=a,
+            )
+
         def test_window_lead_lag(self):
             a = self.make_rand_df(100, a=float, b=(int, 50), c=(str, 50))
             self.eq_sqlite(
@@ -425,6 +507,21 @@ class SQLTests(object):
                 a=a,
             )
 
+        def test_window_lead_lag_partition_by(self):
+            a = self.make_rand_df(100, a=float, b=(int, 50), c=(str, 50))
+            self.eq_sqlite(
+                """
+                SELECT
+                    LEAD(b,1,10) OVER (PARTITION BY c ORDER BY a) AS a3,
+                    LEAD(b,1) OVER (PARTITION BY c ORDER BY b, a ASC NULLS LAST) AS a5,
+
+                    LAG(b,1) OVER (PARTITION BY c ORDER BY a) AS b3,
+                    LAG(b,1) OVER (PARTITION BY c ORDER BY b, a ASC NULLS LAST) AS b5
+                FROM a
+                """,
+                a=a,
+            )
+
         def test_window_sum_avg(self):
             a = self.make_rand_df(100, a=float, b=(int, 50), c=(str, 50))
             for func in ["SUM", "AVG"]:
@@ -445,9 +542,45 @@ class SQLTests(object):
                     """,
                     a=a,
                 )
+                # >= 1.1.0 has bug on these agg function with groupby+rolloing
+                # https://github.com/pandas-dev/pandas/issues/35557
+                if pd.__version__ < "1.1":
+                    # irregular windows
+                    self.eq_sqlite(
+                        f"""
+                        SELECT a,b,
+                            {func}(b) OVER (PARTITION BY b ORDER BY a DESC
+                                ROWS BETWEEN 2 PRECEDING AND 1 PRECEDING) AS a6,
+                            {func}(b) OVER (PARTITION BY b ORDER BY a DESC
+                                ROWS BETWEEN 2 PRECEDING AND 1 FOLLOWING) AS a7,
+                            {func}(b) OVER (PARTITION BY b ORDER BY a DESC
+                                ROWS BETWEEN 2 PRECEDING AND UNBOUNDED FOLLOWING) AS a8
+                        FROM a
+                        """,
+                        a=a,
+                    )
+
+        def test_window_sum_avg_partition_by(self):
+            a = self.make_rand_df(100, a=float, b=(int, 50), c=(str, 50))
+            for func in ["SUM", "AVG"]:
+                self.eq_sqlite(
+                    f"""
+                    SELECT a,b,
+                        {func}(b+a) OVER (PARTITION BY c,b) AS a3,
+                        {func}(b+a) OVER (PARTITION BY b ORDER BY a
+                            ROWS BETWEEN UNBOUNDED PRECEDING AND CURRENT ROW) AS a4,
+                        {func}(b+a) OVER (PARTITION BY b ORDER BY a DESC
+                            ROWS BETWEEN 2 PRECEDING AND CURRENT ROW) AS a5,
+                        {func}(b+a) OVER (PARTITION BY b ORDER BY a
+                            ROWS BETWEEN UNBOUNDED PRECEDING AND UNBOUNDED FOLLOWING)
+                            AS a6
+                    FROM a
+                    """,
+                    a=a,
+                )
                 # 1.1.0 has bug on these agg function with groupby+rolloing
                 # https://github.com/pandas-dev/pandas/issues/35557
-                if not pd.__version__.startswith("1.1.0"):
+                if pd.__version__ < "1.1":
                     # irregular windows
                     self.eq_sqlite(
                         f"""
@@ -484,7 +617,7 @@ class SQLTests(object):
                     a=a,
                 )
                 # < 1.1.0 has bugs on these agg function with rolloing (no group by)
-                if pd.__version__.startswith("1.1.0"):
+                if pd.__version__ >= "1.1":
                     # irregular windows
                     self.eq_sqlite(
                         f"""
@@ -502,7 +635,41 @@ class SQLTests(object):
                 # == 1.1.0 has bugs on these agg function with rolloing (with group by)
                 # https://github.com/pandas-dev/pandas/issues/35557
                 # < 1.1.0 has bugs on nulls when rolling with forward looking
-                if not pd.__version__.startswith("1.1.0"):
+                if pd.__version__ < "1.1":
+                    b = self.make_rand_df(10, a=float, b=(int, 0), c=(str, 0))
+                    self.eq_sqlite(
+                        f"""
+                        SELECT a,b,
+                            {func}(b) OVER (PARTITION BY b ORDER BY a DESC
+                                ROWS BETWEEN 2 PRECEDING AND 1 PRECEDING) AS a6
+                        FROM a
+                        """,
+                        a=b,
+                    )
+
+        def test_window_min_max_partition_by(self):
+            for func in ["MIN", "MAX"]:
+                a = self.make_rand_df(100, a=float, b=(int, 50), c=(str, 50))
+                self.eq_sqlite(
+                    f"""
+                    SELECT a,b,
+                        {func}(b) OVER (PARTITION BY c) AS a2,
+                        {func}(b+a) OVER (PARTITION BY c,b) AS a3,
+                        {func}(b+a) OVER (PARTITION BY b ORDER BY a
+                            ROWS BETWEEN UNBOUNDED PRECEDING AND CURRENT ROW) AS a4,
+                        {func}(b+a) OVER (PARTITION BY b ORDER BY a DESC
+                            ROWS BETWEEN 2 PRECEDING AND CURRENT ROW) AS a5,
+                        {func}(b+a) OVER (PARTITION BY b ORDER BY a
+                            ROWS BETWEEN UNBOUNDED PRECEDING AND UNBOUNDED FOLLOWING)
+                            AS a6
+                    FROM a
+                    """,
+                    a=a,
+                )
+                # >= 1.1.0 has bugs on these agg function with rolloing (with group by)
+                # https://github.com/pandas-dev/pandas/issues/35557
+                # < 1.1.0 has bugs on nulls when rolling with forward looking
+                if pd.__version__ < "1.1":
                     b = self.make_rand_df(10, a=float, b=(int, 0), c=(str, 0))
                     self.eq_sqlite(
                         f"""
@@ -548,7 +715,7 @@ class SQLTests(object):
                 # < 1.1.0 has bugs on these agg function with rolloing (no group by)
                 # == 1.1.0 has this bug
                 # https://github.com/pandas-dev/pandas/issues/35579
-                if pd.__version__.startswith("1.1.0"):
+                if pd.__version__ >= "1.1":
                     # irregular windows
                     self.eq_sqlite(
                         f"""
@@ -567,6 +734,53 @@ class SQLTests(object):
                         a=a,
                     )
 
+        def test_window_count_partition_by(self):
+            for func in ["COUNT"]:
+                a = self.make_rand_df(100, a=float, b=(int, 50), c=(str, 50))
+                self.eq_sqlite(
+                    f"""
+                    SELECT a,b,
+                        {func}(b) OVER (PARTITION BY c) AS a2,
+                        {func}(b+a) OVER (PARTITION BY c,b) AS a3,
+                        {func}(b+a) OVER (PARTITION BY b ORDER BY a
+                            ROWS BETWEEN UNBOUNDED PRECEDING AND CURRENT ROW) AS a4,
+                        {func}(b+a) OVER (PARTITION BY b ORDER BY a DESC
+                            ROWS BETWEEN 2 PRECEDING AND CURRENT ROW) AS a5,
+                        {func}(b+a) OVER (PARTITION BY b ORDER BY a
+                            ROWS BETWEEN UNBOUNDED PRECEDING AND UNBOUNDED FOLLOWING)
+                            AS a6,
+
+                        {func}(c) OVER (PARTITION BY c) AS b2,
+                        {func}(c) OVER (PARTITION BY c,b) AS b3,
+                        {func}(c) OVER (PARTITION BY b ORDER BY a
+                            ROWS BETWEEN UNBOUNDED PRECEDING AND CURRENT ROW) AS b4,
+                        {func}(c) OVER (PARTITION BY b ORDER BY a DESC
+                            ROWS BETWEEN 2 PRECEDING AND CURRENT ROW) AS b5,
+                        {func}(c) OVER (PARTITION BY b ORDER BY a
+                            ROWS BETWEEN UNBOUNDED PRECEDING AND UNBOUNDED FOLLOWING)
+                            AS b6
+                    FROM a
+                    """,
+                    a=a,
+                )
+                # < 1.1.0 has bugs on these agg function with rolloing (no group by)
+                # == 1.1.0 has this bug
+                # https://github.com/pandas-dev/pandas/issues/35579
+                if pd.__version__ >= "1.1":
+                    # irregular windows
+                    self.eq_sqlite(
+                        f"""
+                        SELECT a,b,
+                            {func}(b) OVER (PARTITION BY c ORDER BY a DESC
+                                ROWS BETWEEN 2 PRECEDING AND 0 PRECEDING) AS a9,
+
+                            {func}(c) OVER (PARTITION BY c ORDER BY a DESC
+                                ROWS BETWEEN 2 PRECEDING AND 0 PRECEDING) AS b9
+                        FROM a
+                        """,
+                        a=a,
+                    )
+
         def test_nested_query(self):
             a = self.make_rand_df(100, a=float, b=(int, 50), c=(str, 50))
             self.eq_sqlite(
@@ -661,5 +875,25 @@ class SQLTests(object):
         def test_join_group_having(self):
             pass
 
-        def test_integrated(self):
-            pass
+        def test_integration_1(self):
+            a = self.make_rand_df(
+                100, a=int, b=str, c=float, d=int, e=bool, f=str, g=str, h=float
+            )
+            self.eq_sqlite(
+                """
+                WITH
+                    a1 AS (
+                        SELECT a+1 AS a, b, c FROM a
+                    ),
+                    a2 AS (
+                        SELECT a,MAX(b) AS b_max, AVG(c) AS c_avg FROM a GROUP BY a
+                    ),
+                    a3 AS (
+                        SELECT d+2 AS d, f, g, h FROM a WHERE e
+                    )
+                SELECT a1.a,b,c,b_max,c_avg,f,g,h FROM a1
+                    INNER JOIN a2 ON a1.a=a2.a
+                    LEFT JOIN a3 ON a1.a=a3.d
+                """,
+                a=a,
+            )
